import { Logger } from './utils/logger';
import {
    ClientMCPConfig,
    ChatChunk,
    ConversationState
} from './types';
import OpenAI from "openai";
import {
    Tool as AnthropicTool
} from "@anthropic-ai/sdk/resources/messages/messages.mjs";
import { Client } from "@modelcontextprotocol/sdk/client/index.js";
import { StdioClientTransport } from "@modelcontextprotocol/sdk/client/stdio.js";
import { SSEClientTransport } from "@modelcontextprotocol/sdk/client/sse.js";
import { convertContext } from "./utils/convertContext";

/**
 * Production-ready MCP (Model Context Protocol) client for AI conversations with tool support
 * 
 * @example
 * ```typescript
 * const client = new ClientMCP({
 *   apiKey: "your-api-key",
 *   baseUrl: "https://your-model-context-protocol-server.com",
 *   model: "gemini-2.0-flash",
 *   debug: true
 * });
 * 
 * await client.connectToServer("http://localhost:3003/mcp");
 * 
 * for await (const chunk of client.chat("Hello, how are you?")) {
 *   console.log(chunk.choices[0]?.delta?.content);
 * }
 * ```
 */
export class ClientMCP {
    private static readonly DEFAULT_TIMEOUT = 30000;
    private static readonly DEFAULT_MODEL = "gemini-2.0-flash";
    private static readonly MAX_RECURSION_DEPTH = 5;

    private readonly config: Required<Pick<ClientMCPConfig, 'timeout' | 'debug'>> & {
        apiKey: string;
        model: string;
        baseUrl?: string;
    };

    private readonly logger: Logger;
    private readonly ai: OpenAI;
    private readonly mcp: Client;

    private transport: StdioClientTransport | SSEClientTransport | null = null;
    private tools: AnthropicTool[] = [];
    private functionDeclarations: OpenAI.Chat.Completions.ChatCompletionTool[] = [];
    private messages: OpenAI.Chat.Completions.ChatCompletionMessageParam[] = [];
    private abortController: AbortController;
    private isConnected = false;
    private recentConversationLength = 0;

    constructor(config: ClientMCPConfig) {
        this.validateConfig(config);
        this.config = this.normalizeConfig(config);
        this.logger = Logger.getInstance(this.config.debug);
        this.ai = this.initializeOpenAI();
        this.mcp = this.initializeMCP();
        this.abortController = new AbortController();
        this.addMessage(config.systemMessages || `# AI Assistant System Prompt

            You are an intelligent AI assistant with access to a comprehensive toolkit. Your primary objective is to successfully complete user tasks through strategic planning and tool utilization.
            
            ## Core Principles
            
            **Strategic Planning**: Begin every task by creating a clear, step-by-step plan. Outline your approach, identify required tools, and anticipate potential challenges.
            
            **Adaptive Problem-Solving**: Monitor your progress continuously. If your initial plan encounters obstacles or proves ineffective, reassess and refactor your approach immediately.
            
            **Tool Mastery**: You have access to multiple tools that can be used individually or in combination. Leverage these tools strategically to maximize efficiency and effectiveness.
            
            ## Operational Guidelines
            
            1. **Never decline a task prematurely** - Always explore available tools and search for additional capabilities first
            2. **Plan before execution** - Create a structured approach for every task, no matter how simple
            3. **Execute systematically** - Follow your plan while remaining flexible to adjust based on results
            4. **Utilize parallel processing** - When possible, use multiple tools simultaneously to improve efficiency
            5. **Document your reasoning** - Explain your planning decisions and tool choices to ensure transparency
            
            ## Success Criteria
            
            Your success is measured by task completion, not by the complexity of your approach. Prioritize finding solutions over explaining limitations. If a direct path isn't apparent, explore alternative approaches through tool discovery and creative problem-solving.
            
            Remember: Every task has a potential solution - your job is to find it through intelligent planning and resourceful tool utilization.`,
                        "system");

    }

    /**
     * Establishes connection to MCP server
     * 
     * @param serverPath - URL or string path to the MCP server
     * @param sessionId - Optional session ID for reconnection
     */
    public async connectToServer(serverPath: string | URL, sessionId?: string): Promise<void> {
        if (this.isConnected) {
            this.logger.warn('Already connected to MCP server');
            return;
        }

        try {
            this.transport = await this.createTransport(serverPath, sessionId);
            await this.mcp.connect(this.transport);
            await this.loadTools();
            this.isConnected = true;
            this.logger.info(`Successfully connected to MCP server: ${serverPath}`);
        } catch (error) {
            this.logger.error('Failed to connect to MCP server:', error);
            throw new Error(`MCP connection failed: ${error instanceof Error ? error.message : 'Unknown error'}`);
        }
    }

    /**
     * Disconnects from MCP server
     */
    public async disconnect(): Promise<void> {
        if (!this.isConnected) return;

        try {
            if (this.transport) {
                await this.transport.close();
                this.transport = null;
            }
            this.isConnected = false;
            this.logger.info('Disconnected from MCP server');
        } catch (error) {
            this.logger.error('Error during disconnect:', error);
        }
    }

    /**
     * Performs streaming chat completion
     * 
     * @param options - Options for the chat completion
     * @returns Async generator of chat completion chunks
     */
    public async *chatCompletionStream(options: {
        messages: OpenAI.Chat.Completions.ChatCompletionMessageParam[];
        reasoningEffort?: "low" | "medium" | "high";
        tools?: OpenAI.Chat.Completions.ChatCompletionTool[];
        toolChoice?: OpenAI.Chat.Completions.ChatCompletionCreateParams['tool_choice'];
    }): AsyncGenerator<OpenAI.Chat.Completions.ChatCompletionChunk> {
        try {
            const stream = await this.ai.chat.completions.create({
                model: this.config.model,
                messages: options.messages,
                ...(options.reasoningEffort && { reasoning_effort: options.reasoningEffort }),
                ...(options.tools && { tools: options.tools }),
                ...(options.toolChoice && { tool_choice: options.toolChoice }),
                stream: true,
            });

            for await (const chunk of stream) {
                yield chunk;
            }
        } catch (error) {
            this.logger.error('Streaming chat completion failed:', error);
            throw this.handleApiError(error);
        }
    }

    /**
     * Main chat interface with automatic tool handling and conversation management
     * 
     * @param content - User input message
     * @param options - Options for the chat with maxDepth and autoSummarize
     * Where maxDepth is the maximum depth of recursion and autoSummarize is whether to automatically summarize the conversation
     * @returns Async generator of chat completion chunks and tool calls
     */
    public async *chat(
        content: string,
        options: {
            maxDepth?: number;
            autoSummarize?: boolean;
        } = {}
    ): AsyncGenerator<OpenAI.Chat.Completions.ChatCompletionChunk | ChatChunk> {
        const { maxDepth = ClientMCP.MAX_RECURSION_DEPTH, autoSummarize = false } = options;
        this.recentConversationLength = 0;
        yield* this.chatInternal(content, 0, maxDepth, autoSummarize);
    }

    /**
     * Cancels all ongoing requests
     */
    public cancelRequests(): void {
        this.abortController.abort();
        this.abortController = new AbortController();
        this.logger.info('All requests cancelled');
    }

    /**
     * Gets current conversation state
     * 
     * @returns Current conversation state
     */
    public getConversationState(): ConversationState {
        return {
            messageCount: this.messages.length,
            toolsAvailable: this.tools.length,
            isConnected: this.isConnected,
            lastActivity: new Date()
        };
    }

    /**
     * Clears conversation history
     */
    public clearConversation(): void {
        const systemMessages = this.messages.filter(msg => msg.role === 'system');
        this.messages = systemMessages;
        this.logger.info('Conversation history cleared');
    }

    // Private methods

    private validateConfig(config: ClientMCPConfig): void {
        if (!config.apiKey?.trim()) {
            throw new Error('API key is required and cannot be empty');
        }
    }

    private normalizeConfig(config: ClientMCPConfig) {
        return {
            timeout: config.timeout ?? ClientMCP.DEFAULT_TIMEOUT,
            debug: config.debug ?? false,
            apiKey: config.apiKey.trim(),
            model: config.model ?? ClientMCP.DEFAULT_MODEL,
            baseUrl: config.baseUrl
        };
    }

    private initializeOpenAI(): OpenAI {
        return new OpenAI({
            apiKey: this.config.apiKey,
            ...(this.config.baseUrl && { baseURL: this.config.baseUrl }),
            timeout: this.config.timeout
        });
    }

    private initializeMCP(): Client {
        return new Client(
            { name: "mcp-client-cli", version: "1.0.0" },
            { capabilities: { tools: {} } }
        );
    }

    private async createTransport(
        serverPath: string | URL,
        sessionId?: string
    ): Promise<StdioClientTransport | SSEClientTransport> {
        if (typeof serverPath === 'string') {
            return this.createStdioTransport(serverPath);
        } else {
            return this.createSSETransport(serverPath, sessionId);
        }
    }

    private createStdioTransport(serverPath: string): StdioClientTransport {
        const isJavaScript = serverPath.endsWith('.js');
        const isPython = serverPath.endsWith('.py');

        if (!isJavaScript && !isPython) {
            throw new Error('Server script must be a .js or .py file');
        }

        const command = isPython
            ? (process.platform === "win32" ? "python" : "python3")
            : process.execPath;

        return new StdioClientTransport({
            command,
            args: [serverPath]
        });
    }

    private createSSETransport(serverUrl: URL, sessionId?: string): SSEClientTransport {
        return new SSEClientTransport(serverUrl, {
            requestInit: {
                headers: {
                    'Accept': 'text/event-stream',
                    'Content-Type': 'application/json',
                    ...(sessionId && { 'mcp-session-id': sessionId })
                }
            }
        });
    }

    private async loadTools(): Promise<void> {
        const toolsResult = await this.mcp.listTools();

        this.tools = toolsResult.tools.map(tool => ({
            name: tool.name,
            description: tool.description,
            input_schema: tool.inputSchema
        }));

        this.functionDeclarations = convertContext(this.tools);

        this.logger.info(`Loaded ${this.tools.length} tools:`,
            this.tools.map(tool => `${tool.name}: ${tool.description}`).join(', ')
        );
    }


    private async *chatInternal(
        content: string,
        depth: number,
        maxDepth: number,
        autoSummarize: boolean
    ): AsyncGenerator<OpenAI.Chat.Completions.ChatCompletionChunk | ChatChunk> {
        if (depth >= maxDepth) {
            yield* this.handleMaxDepthReached(content, maxDepth);
            return;
        }

        this.addMessage(content, "user");
        let assistantResponse = '';

        // Stream the main conversation
        for await (const chunk of this.chatCompletionStream({
            messages: this.messages,
            tools: this.functionDeclarations.length > 0 ? this.functionDeclarations : undefined
        })) {
            const textContent = chunk.choices[0]?.delta?.content || '';
            if (textContent) {
                assistantResponse += textContent;
                yield chunk;
            }

            // Handle tool calls
            const toolCalls = chunk.choices[0]?.delta?.tool_calls;
            if (toolCalls?.length) {
                for (const toolCall of toolCalls) {
                    yield {
                        choices: [{
                            delta: {
                                content: JSON.stringify({
                                    tool: toolCall.function?.name,
                                    args: JSON.parse(toolCall.function?.arguments || "{}"),
                                    executed: false
                                }, null, 2),
                                tool_calls: [{
                                    function: {
                                        name: toolCall.function?.name!,
                                        arguments: toolCall.function?.arguments!
                                    }
                                }]
                            }
                        }]
                    }
                }
                const toolResults = await this.executeToolCalls(toolCalls);
                for (const result of toolResults) {
                    autoSummarize = true;
                    if (result) yield result;
                }
            }
        }

        if (assistantResponse.trim()) {
            this.addMessage(assistantResponse.trim(), 'assistant');
        }

        // Check if continuation is needed
        const shouldContinue = await this.shouldContinueConversation();
        if (shouldContinue.continue && shouldContinue.nextMessage) {
            yield* this.chatInternal(shouldContinue.nextMessage, depth + 1, maxDepth, autoSummarize);
        } else if (autoSummarize) {
            yield* this.generateConversationSummary();
        }
    }

    private async *handleMaxDepthReached(
        originalContent: string,
        maxDepth: number
    ): AsyncGenerator<OpenAI.Chat.Completions.ChatCompletionChunk> {
        this.logger.warn(`Max depth (${maxDepth}) reached for conversation`);

        const contextMessage = `Maximum conversation depth reached. Original request: "${originalContent}". ` +
            `Please provide a final response based on the conversation context or indicate if additional input is needed.`;

        yield* this.chatCompletionStream({
            messages: [...this.messages, { role: "user", content: contextMessage }]
        });
    }

    private async executeToolCalls(
        toolCalls: OpenAI.Chat.Completions.ChatCompletionChunk.Choice.Delta.ToolCall[]
    ): Promise<(ChatChunk | null)[]> {
        const toolPromises = toolCalls
            .filter(call => call.function?.name && call.function?.arguments)
            .map(call => this.executeToolCall(call));

        return Promise.all(toolPromises);
    }

    private async executeToolCall(
        toolCall: OpenAI.Chat.Completions.ChatCompletionChunk.Choice.Delta.ToolCall
    ): Promise<ChatChunk | null> {
        const { name, arguments: args } = toolCall.function!;

        try {
            const toolArgs = JSON.parse(args || "{}");
            const result = await this.mcp.callTool({ name: name!, arguments: toolArgs });

            const resultContent = this.formatToolResult(name!, toolArgs, result.content);
            this.addMessage(resultContent, 'assistant');

            return {
                choices: [{
                    delta: {
                        content: resultContent,
                        tool_calls: [{ function: { name: name!, arguments: args || "{}" } }]
                    }
                }]
            };
        } catch (error) {
            this.logger.error(`Tool execution failed for ${name}:`, error);
            const errorContent = `Tool "${name}" execution failed: ${error instanceof Error ? error.message : 'Unknown error'}`;

            return {
                choices: [{
                    delta: {
                        content: errorContent,
                        tool_calls: [{ function: { name: name!, arguments: args || "{}" } }]
                    }
                }]
            };
        }
    }

    private formatToolResult(name: string, args: Record<string, unknown>, result: unknown): string {
        const resJson = {
            tool: name,
            args,
            result,
            executed: true
        }
        return JSON.stringify(resJson, null, 2);
    }

    private async shouldContinueConversation(): Promise<{ continue: boolean; nextMessage?: string }> {
        try {
            const tools: OpenAI.Chat.Completions.ChatCompletionTool[] = [{
                "type": "function",
                "function": {
                    "name": "evaluate_conversation_status",
                    "description": `Analyze the current conversation state and determine task completion status.
This function evaluates whether the user's original request has been fully satisfied or if additional work is required. It performs a comprehensive assessment of:
- Task completion status against original requirements
- Quality and completeness of delivered solutions
- Remaining steps or subtasks that need attention
- Logical next actions to move toward full completion

If the task is incomplete, the function identifies the most appropriate next step and provides a clear message to continue the workflow. The suggested next steps should be actionable using the available tools.

Available tools for next steps:\n` + this.functionDeclarations.map((tool, index) => `${index + 1}. ${tool.function.name}: ${tool.function.description}`).join('\n'),
                    "parameters": {
                        "type": "object",
                        "properties": {
                            "continue": {
                                "type": "boolean",
                                "description": "Whether additional steps are needed to complete the user's original request. Set to true if any tasks, refinements, or follow-up actions remain unfinished."
                            },
                            "nextMessage": {
                                "type": "string",
                                "description": "The next actionable message or instruction to process if continuation is needed. Should be specific, clear, and executable using available tools to advance task completion."
                            }
                        },
                        "required": ["continue"]
                    }
                }
            }];

            const evaluationPrompt = "Have you completed the user's original request? If not, suggest the next logical step.";

            let functionArgs = '';
            for await (const chunk of this.chatCompletionStream({
                messages: [...this.messages, { role: "user", content: evaluationPrompt }],
                tools
            })) {
                const toolCall = chunk.choices[0]?.delta?.tool_calls?.[0];
                if (toolCall?.function?.name === "evaluate_conversation_status") {
                    functionArgs += toolCall.function.arguments || '';
                }
            }

            const result = JSON.parse(functionArgs || '{"continue": false}');
            return {
                continue: result.continue || false,
                nextMessage: result.nextMessage
            };
        } catch (error) {
            this.logger.error('Error evaluating conversation status:', error);
            return { continue: false };
        }
    }

    private async *generateConversationSummary(): AsyncGenerator<OpenAI.Chat.Completions.ChatCompletionChunk> {
        const summaryPrompt = "Please provide a concise conclusion what user initially asked, " +
        "highlighting the key points, decisions made, and any important outcomes." +
        `Do not use summary word in the response.
        Instructions:   
        1. Just provide the conclusion and proper response what user asked.
        2. Read the tools response and create the response based on that.
        3. Do not provide any additional information.
        4. **Important**: Your response will be shown to the user, so be concise and provide the proper response.
        `;
        this.logger.debug("messages::", [...this.messages, { role: "user", content: summaryPrompt }]);
        yield* this.chatCompletionStream({
            messages: [...this.messages, { role: "user", content: summaryPrompt }]
        });
    }

    private addMessage(content: string, role: "user" | "assistant" | "system"): void {
        this.messages.push({ role, content });
        this.recentConversationLength = this.recentConversationLength + 1;
    }

    private handleApiError(error: unknown): Error {
        if (error instanceof Error) {
            if (error.name === 'AbortError') {
                return new Error('Request was cancelled');
            }
            return error;
        }
        return new Error('Unknown API error occurred');
    }



}