/**
 * Base error class for Gemini MCP client errors
 */
export class ClientMCPError extends Error {
  constructor(message: string, public code: string, public details?: unknown) {
    super(message);
    this.name = 'ClientMCPError';
    
    // Maintains proper stack trace for where our error was thrown
    if (Error.captureStackTrace) {
      Error.captureStackTrace(this, ClientMCPError);
    }
  }
}

/**
 * Error thrown when API request fails
 */
export class APIError extends ClientMCPError {
  constructor(
    message: string, 
    public status: number, 
    code: string = 'API_ERROR', 
    details?: unknown
  ) {
    super(message, code, details);
    this.name = 'APIError';
  }
}

/**
 * Error thrown when authentication fails
 */
export class AuthenticationError extends ClientMCPError {
  constructor(message: string = 'Authentication failed', details?: unknown) {
    super(message, 'AUTH_ERROR', details);
    this.name = 'AuthenticationError';
  }
}

/**
 * Error thrown when a resource is not found
 */
export class NotFoundError extends ClientMCPError {
  constructor(resource: string, id?: string) {
    const message = id 
      ? `${resource} with ID ${id} not found`
      : `${resource} not found`;
    super(message, 'NOT_FOUND');
    this.name = 'NotFoundError';
  }
}

/**
 * Error thrown when a request is invalid
 */
export class ValidationError extends ClientMCPError {
  constructor(message: string = 'Invalid request', details?: unknown) {
    super(message, 'VALIDATION_ERROR', details);
    this.name = 'ValidationError';
  }
}

/**
 * Error thrown when a request times out
 */
export class TimeoutError extends ClientMCPError {
  constructor(timeout: number) {
    super(`Request timed out after ${timeout}ms`, 'TIMEOUT_ERROR');
    this.name = 'TimeoutError';
  }
}

/**
 * Error thrown when the API rate limit is exceeded
 */
export class RateLimitError extends ClientMCPError {
  constructor(
    message: string = 'Rate limit exceeded', 
    public retryAfter?: number
  ) {
    super(message, 'RATE_LIMIT_EXCEEDED');
    this.name = 'RateLimitError';
  }
}
